package org.processmining.logfiltering.visualizerModel;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.Pair;
import org.processmining.logfiltering.visConstants.ModelConstants;
import org.processmining.logfiltering.visFilteringAlgorithms.ApplyFilteringAlgorithm;
import org.processmining.logfiltering.visFilteringAlgorithms.FilteringAlgorithmFactory;
import org.processmining.logfiltering.visFilteringAlgorithms.FilteringAlgorithmParameters;
import org.processmining.logfiltering.visMiningAlgorithms.ApplyMiningAlgorithm;
import org.processmining.logfiltering.visMiningAlgorithms.MiningAlgorithmParameters;
import org.processmining.logfiltering.visMiningAlgorithms.MiningAlgorithmsFactory;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * Model that is stored as ProM object and after that is shown by a custom visualizer
 * @author berti
 *
 */
public class FilteringVisualizerModel {
	// the plugin execution context
	PluginContext context;
	// the original log that is being filtered
	XLog originalLog;
	// the filtered log (for export and mining purposes)
	XLog filteredLog;
	// The Petrinet obtained by the algorithm
	Petrinet net;
	
	public XLog reportTraceAttributesIntoEvents(XLog log) {
		for (XTrace trace : log) {
			XAttributeMap traceAttributeMap = trace.getAttributes();
			
			for (XEvent event : trace) {
				XAttributeMap eventAttributeMap = event.getAttributes();
				
				for (String traceAttribute : traceAttributeMap.keySet()) {
					if (!eventAttributeMap.containsKey(traceAttribute)) {
						//System.out.println("replicating attribute "+traceAttribute+" in trace "+trace.getAttributes().get("concept:name").toString()+" to event with hashcode "+event.hashCode());
						
						// we enter here if the trace attribute is not also an attribute of the event
						// (so we don't replicate concept:name that in trace means the case ID and in events means the activity)
						
						XAttribute traceAttributeValue = traceAttributeMap.get(traceAttribute);
						eventAttributeMap.put(traceAttribute, traceAttributeValue);
						
					}
				}
			}
		}

		return log;
	}
	
	public FilteringVisualizerModel(PluginContext context, XLog originalLog) {
		this.context = context;
		
		this.originalLog = originalLog;
		this.originalLog = reportTraceAttributesIntoEvents(this.originalLog);
		// at the start no filtering is applied
		this.filteredLog = originalLog;
		// Petri net has not been mined
		this.net = null;
		
		// apply the default algorithms at the end of this constructor
		this.filterLogAndGetPetrinet();
	}
	
	public XLog getOriginalLog() {
		return originalLog;
	}

	public void setOriginalLog(XLog originalLog) {
		this.originalLog = originalLog;
	}

	public XLog getFilteredLog() {
		return filteredLog;
	}

	public void setFilteredLog(XLog filteredLog) {
		this.filteredLog = filteredLog;
	}

	public Petrinet getNet() {
		return net;
	}

	public void setNet(Petrinet net) {
		this.net = net;
	}
	
	public void filterLogAndGetPetrinet() {
		// by default apply Alpha Miner
		MiningAlgorithmsFactory.algorithms miningAlgorithmDescription = ModelConstants.defaultMiningAlgorithm;
		filterLogAndGetPetrinet(miningAlgorithmDescription);
	}
	
	public void filterLogAndGetPetrinet(MiningAlgorithmsFactory.algorithms miningAlgorithmDescription) {
		// calls the method to do the mining allocating the default parameters
		MiningAlgorithmsFactory miningFactory = new MiningAlgorithmsFactory();
		ApplyMiningAlgorithm miningAlgorithm = miningFactory.getMiningAlgorithm(miningAlgorithmDescription);
		filterLogAndGetPetrinet(miningAlgorithm);
	}
	
	public void filterLogAndGetPetrinet(ApplyMiningAlgorithm miningAlgorithm) {
		FilteringAlgorithmFactory filteringFactory = new FilteringAlgorithmFactory();
		ApplyFilteringAlgorithm filteringAlgorithm = filteringFactory.getFilteringAlgorithm(ModelConstants.defaultFilteringMethod);
		filterLogAndGetPetrinet(miningAlgorithm, filteringAlgorithm);
	}
	
	public void filterLogAndGetPetrinet(ApplyMiningAlgorithm miningAlgorithm, ApplyFilteringAlgorithm filteringAlgorithm) {
		filterLogAndGetPetrinet(miningAlgorithm, filteringAlgorithm, null, null);
	}
	
	public void filterLogAndGetPetrinet(ApplyMiningAlgorithm miningAlgorithm, ApplyFilteringAlgorithm filteringAlgorithm, MiningAlgorithmParameters algorithmParameters, FilteringAlgorithmParameters filteringParameters) {
		// some tricks are used here to avoid duplicating code: if the parameters provided in the method are null then
		// the default options are applied :)
		if (filteringParameters != null) {
			this.filteredLog = filteringAlgorithm.getFilteredLog(this.context, this.originalLog, filteringParameters);
		}
		else {
			this.filteredLog = filteringAlgorithm.getFilteredLog(this.context, this.originalLog);
		}
		
		Pair<Petrinet, Marking> result;
		if (algorithmParameters != null) {
			result = miningAlgorithm.getPetrinet(this.context, this.filteredLog, algorithmParameters);
		}
		else {
			result = miningAlgorithm.getPetrinet(this.context, this.filteredLog);
		}
		this.net = result.getFirst();
		
		// some debug prints
		System.out.println("originalLog size = "+this.originalLog.size());
		System.out.println("filteredLog size = "+this.filteredLog.size());
		System.out.println("net places size = "+this.net.getPlaces().size());
	}
}
